function [tout,yout] = eEuler(f,tspan,y0,h)
%% Euler-Vorw�rts Algorithmus
% [tout,yout] = eEuler(f,tspan,y0,h)
%
% Numerische Methoden der Dynamik - SS19
% Dr.-Ing. Pascal Ziegler
%
% Institut f�r Technische und Numerische Mechanik
% Prof. Dr.-Ing. Prof. E.h. P. Eberhard
%
% Autor: Andreas Hofmann, M.Sc.
%

if nargin < 4
    h = 0.01; % default Schrittweite
end

% Simulationszeit
t0 = tspan(1); % Start
t1 = tspan(2); % Ende

[timeVec, adapted] = simTime2vector(t0,t1,h);

% Anfangsbedingungen
y = y0;

% R�ckgabe & Speichern der Anfangswerte
yout = NaN(size(y,1),length(timeVec));
tout = NaN(1,length(timeVec));


% while t < t1-eps   h = 0.01 t1 = 0.1
for index = 1:length(timeVec)
    
    t = timeVec(index);
    if adapted && (index == length(timeVec)-1)
        h = timeVec(index+1) - timeVec(index);
    end
    
    % Aktuelle Werte speichern
    tout(:,index) = t;
    yout(:,index) = y;
    
    % Schritt:
    yNeu = y + h * f(t,y);  % y(i+1) = y(i) + h * f(t(i),y(i))
                            % d/dt(y) = f(t(i),y(i))
    y = yNeu;    
end

end
