%% A07 Loesung linearer Gleichungssysteme mit Gauss
%
% Numerische Methoden der Dynamik - SS19
% Dr.-Ing. Pascal Ziegler
%
% Institut fuer Technische und Numerische Mechanik
% Prof. Dr.-Ing. Prof. E.h. P. Eberhard
%
% Autor: Andreas Hofmann, M.Sc.
%

clear all
close all

% Notwendige Ordner zum Matlabpfad hinzufuegen
nameFile = mfilename('fullpath');
pathFile = fileparts(nameFile);
pathLGS = fullfile(pathFile,'lgsSolve');
addpath(pathLGS)

%% Beispiel A7.2
% Kleines gut definiertes System;
A = [1 2 2; 3 5 1; 2 6 5];
b = [3; -4; 5];

% Gauss Algorithmus ohne Pivotsuche
[XohnePivot, TohnePivot] = GaussOhnePivot(A,b);

% Gauss Algorithmus mit Pivotsuche
[XmitPivot, TmitPivot] = GaussMitPivot(A,b);

% Matlab built-in solver
tstart = tic;
x = linsolve(A,b); % siehe Hilfe linsolve
Tlinsolve = toc(tstart);

tstart = tic;
Xmldivide = A\b; % siehe Hilfe mldivide
Tmldivide = toc(tstart);

%% Beispiel A7.4
% Rundende Dezimalarithmetik

clear all

% Initialisierung des LGS
A = [1 1 1; 1 1.001 5; 1 2 2];
b = [1; 2; 1];

Xexakt = GaussOhnePivot(A,b,'Toleranz'); % Exakte Loesung (mit Standardgenauigkeit)

% 4 stellig-rundende Dezimalarithmetik
global precisionControl; precisionControl = true;
currentDigits = digits;
digits(4) % 4 Stellen

XohnePivot4Dec = GaussOhnePivotPrec(A,b,'Toleranz',1e-6);
XmitPivot4Dec = GaussMitPivotPrec(A,b);

digits(currentDigits); % Wiederherstellung der Default-Genauigkeit


% Darstellung der Ergebnisse in Einer Tablle
% relativer Fehler = ||Xnum - Xexakt|| / ||Xexakt|| *100%
relFehlOhnePivot = abs(XohnePivot4Dec - Xexakt)./abs(Xexakt) *100;
relFehlMitPivot =  abs(XmitPivot4Dec - Xexakt)./abs(Xexakt) *100;

table(['x1';'x2';'x3'],Xexakt,...
    XohnePivot4Dec,relFehlOhnePivot,...
    XmitPivot4Dec,relFehlMitPivot ,...
    'VariableNames',{'X' 'exakt' 'ohnePivot' 'relFehler' 'mitPivot' 'relFehlerPiv'})



