
function [X, tElapsed] = GaussMitPivotPrec(A,B,varargin)
%% LGS Loeser mit Gauss-Elimination mit Pivotsuche vgl. A7.3 (Dezimalarithmetik)
% [X, tElapsed] = GaussMitPivotPrec(A,B)
%
% Numerische Methoden der Dynamik - SS19
% Dr.-Ing. Pascal Ziegler
%
% Institut fuer Technische und Numerische Mechanik
% Prof. Dr.-Ing. Prof. E.h. P. Eberhard
%
% Autor: Andreas Hofmann, M.Sc.
%

% Default Toleranz
epsilon = 1e-6;

if ~isempty(varargin)
    if strcmp(regexp(varargin{1}),'toleranz')
        epsilon = varargin{2}';
    else
        warning(['Eingabeoptionen prüfen!',...
            ' Keine spezifische Toleranz definiert.',...
            ' Berechnung mit dem Default-Wert: ',num2str(epsilon)])
    end
end

%% 1. Untersuchung der übergebenen Werte
dimA = size(A); % dimA = [nZeilen nSpalten]; A(n x n)
dimB = size(B); % dimB = [nZeilen qSpalten]; B(n x q)

if dimA(1) ~= dimA(2)
    error('Matrix A is not quadratic!')
elseif dimA(1) ~= dimB(1)
    error('Dimensions of A and B do not match')
end

n = dimA(1);
q = dimB(2);

A = myPrecision(A); % rundende Dezimalarithmetik mit myPrecision()
B = myPrecision(B);

%% 2. Gauss-Elimination mit Pivotsuche
tStart = tic;

for s = 1:1:(n-1)
    
    % Pivotsuche
    [piv, idx] = max(A(s:n,s));
    p = (s-1)+idx;
    
    if piv <= epsilon
        error('A ist singulaer')
    end
    
    % Zeilenvertauschen
    if p ~= s
        % Matrix A
        l = A(s,:);
        A(s,:) = A(p,:);
        A(p,:) = l;

        %Matrix B
        l = B(s,:);
        B(s,:) = B(p,:);
        B(p,:) = l;  
    end
    
    % Elimination
    for i = (s+1):1:n
        l = myPrecision(A(i,s)/A(s,s));
        A(i,s) = 0;
        
        for j = (s+1):1:n
            A(i,j) = myPrecision(A(i,j) - l * A(s,j));
        end
        
        for k = 1:1:q
            B(i,k) = myPrecision(B(i,k) - l * B(s,k));
        end
    end
end

if myPrecision(abs(A(n,n))) <= epsilon
    error ('A ist singulaer')
end

%% Loesung durch Rueckwaertseinsetzen
R = A;
C = B;
X = RueckwaertsEinsetzen(R,C);

tElapsed = toc(tStart);
end